@@ -2,6 +2,7 @@ package DJabberd::Connection;
 use strict;
 use warnings;
 use base 'Danga::Socket';
+use bytes;
 use fields (
             'saxhandler',
             'parser',
@@ -12,6 +13,7 @@ use fields (
             'server',         # our DJabberd server object, which we used to find the VHost
             'ssl',            # undef when not in ssl mode, else the $ssl object from Net::SSLeay
             'stream_id',      # undef until set first time
+            'to_host',        # undef until stream start
             'version',        # the DJabberd::StreamVersion we negotiated
             'rcvd_features',  # the features stanza we've received from the other party
             'log',            # Log::Log4perl object for this connection
@@ -23,6 +25,7 @@ use fields (
             'in_stream',      # bool:  true if we're in a stream tag
             'counted_close',  # bool:  temporary here to track down the overcounting of disconnects
             'disconnect_handlers',  # array of coderefs to call when this connection is closed for any reason
+
             );
 
 our $connection_id = 1;
@@ -63,8 +66,8 @@ sub new {
     my ($class, $sock, $server) = @_;
     my $self = $class->SUPER::new($sock);
 
-    croak("Server param not a DJabberd (server) object, actually a '$server'")
-        unless ref $server eq "DJabberd";
+    croak("Server param not a DJabberd (server) object, '" . ref($server) . "'")
+        unless $server->isa("DJabberd");
 
     $self->{vhost}   = undef;  # set once we get a stream start header from them.
     $self->{server}  = $server;
@@ -88,7 +91,7 @@ sub new {
     }
 
     if (XMLDEBUG) {
-        system("mkdir -p " . XMLDEBUG ."$$/");
+        system("mkdir -p " . XMLDEBUG ."/$$/");
         my $handle = IO::Handle->new;
         no warnings;
         my $from = $fromip || "outbound";
@@ -249,6 +252,17 @@ sub set_bound_jid {
     $self->{bound_jid} = $jid;
 }
 
+sub set_to_host {
+    my ($self, $host) = @_;
+    $self->{to_host} = $host;
+}
+
+sub to_host {
+    my $self = shift;
+    return $self->{to_host} or
+        die "To host accessed before it was set";
+}
+
 sub set_version {
     my ($self, $verob) = @_;
     $self->{version} = $verob;
@@ -484,7 +498,20 @@ sub event_read {
 
         # Net::SSLeays buffers internally, so if we didn't read anything, it's
         # in its buffer
-        return unless $data && length $data;
+        unless ($data && length $data) {
+            # a few of these in a row implies an EOF.  else it could
+            # just be the underlying socket was readable, but there
+            # wasn't enough of an SSL packet for OpenSSL/etc to return
+            # any unencrypted data back to us.
+            # We call 'actual_error_on_empty_read' to avoid counting
+            # SSL_ERROR_WANT_READ or SSL_ERROR_WANT_WRITE as 'actual' errors
+            my $err = DJabberd::Stanza::StartTLS->actual_error_on_empty_read($ssl);
+            if ($err) {
+                $self->log->warn("SSL Read error: $err (assuming ssl_eof)");
+                $self->close('ssl_eof');
+            }
+            return;
+        }
         $bref = \$data;
     } else {
         # non-ssl mode:
@@ -581,6 +608,7 @@ sub start_stream_back {
 
     # bind us to a vhost now.
     my $to_host     = $ss->to;
+    $self->set_to_host($to_host) if $to_host;
 
     # Spec rfc3920 (dialback section) says: Note: The 'to' and 'from'
     # attributes are OPTIONAL on the root stream element.  (during
@@ -622,6 +650,17 @@ sub start_stream_back {
             && !$self->isa("DJabberd::Connection::ServerIn")) {
             $features_body .= "<starttls xmlns='urn:ietf:params:xml:ns:xmpp-tls' />";
         }
+        if( $self->vhost ){
+          $self->vhost->hook_chain_fast("SendFeatures",
+                                        [ $self ],
+                                        {
+                                            stanza => sub {
+                                              my ($self, $xml_string) = @_;
+                                              $features_body .= $xml_string;
+                                            },
+                                        }
+                                        );          
+        }
         $features = qq{<stream:features>$features_body</stream:features>};
     }
 
@@ -751,11 +790,6 @@ sub close {
     $self->log->debug("DISCONNECT: $self->{id}\n") if $self->{id};
     $self->_run_callback_list($self->{disconnect_handlers});
 
-    if (my $ssl = $self->{ssl}) {
-        Net::SSLeay::free($ssl);
-        $self->{ssl} = undef;
-    }
-
     if (my $p = $self->{parser}) {
         # libxml isn't reentrant apparently, so we can't finish_push
         # from inside an existint callback.  so schedule immediately,
