@@ -102,8 +102,10 @@ sub send_reply {
     $raw ||= "";
     my $id = $self->id;
     my $bj = $conn->bound_jid;
-    my $to = $bj ? ("to='" . $bj->as_string_exml . "'") : "";
-    my $xml = qq{<iq $to type='$type' id='$id'>$raw</iq>};
+    my $from_jid = $self->to;
+    my $to = $bj ? (" to='" . $bj->as_string_exml . "'") : "";
+    my $from = $from_jid ? (" from='" . $from_jid . "'") : "";
+    my $xml = qq{<iq$to$from type='$type' id='$id'>$raw</iq>};
     $conn->xmllog->info($xml);
     $conn->write(\$xml);
 }
@@ -502,9 +504,17 @@ sub process_iq_setauth {
 
 
     my $accept = sub {
+        my $cb = shift;
+        my $authjid = shift;
+
+        # create default JID
+        unless (defined $authjid) {
+            my $sname = $vhost->name;
+            $authjid = "$username\@$sname";
+        }
+
         # register
-        my $sname = $vhost->name;
-        my $jid = DJabberd::JID->new("$username\@$sname/$resource");
+        my $jid = DJabberd::JID->new("$authjid/$resource");
 
         unless ($jid) {
             $reject->();
@@ -529,9 +539,13 @@ sub process_iq_setauth {
         $vhost->register_jid($jid, $conn, $regcb);
     };
 
-    my $can_get_password = $vhost->are_hooks("GetPassword");
 
-    if ($can_get_password) {
+    # XXX FIXME
+    # If the client ignores your wishes get a digest or password
+    # We should throw an error indicating so
+    # Currently we will just return authentication denied -- artur
+
+    if ($vhost->are_hooks("GetPassword")) {
         $vhost->run_hook_chain(phase => "GetPassword",
                               args  => [ username => $username, conn => $conn ],
                               methods => {
