@@ -1,5 +1,7 @@
 BEGIN {
     $ENV{LOGLEVEL} ||= "WARN";
+    use DJabberd::Log;
+    DJabberd::Log->set_logger();
 }
 use strict;
 use DJabberd;
@@ -165,6 +167,9 @@ use strict;
 use overload
     '""' => \&as_string;
 
+use Data::Dumper    qw[Dumper];
+local $Data::Dumper::Indent = 1;
+
 our $PLUGIN_CB;
 our $VHOST_CB;
 our @SUBDOMAINS;
@@ -178,7 +183,7 @@ sub new {
     my $class = shift;
     my $self = bless {@_}, $class;
 
-    die unless $self->{id};
+    die 'ID required. Pass it as ->new(id => $id)' unless $self->{id};
     return $self;
 }
 
@@ -276,7 +281,11 @@ sub start {
         $server->set_config_serverport($self->serverport);
         $server->set_config_clientport($self->clientport);
         $server->set_config_adminport($self->adminport) if $self->adminport;
-
+        
+        if( my $server_callback = shift ){
+          $server_callback->($server);
+        }
+        
         my $childpid = fork;
         if (!$childpid) {
             $server->run;
@@ -314,6 +323,9 @@ use strict;
 use overload
     '""' => \&as_string;
 
+use Data::Dumper    qw[Dumper];
+local $Data::Dumper::Indent = 1;
+
 sub resource {
     return $_[0]{resource} ||= ($ENV{UNICODE_RESOURCE} ? "test\xe2\x80\x99s computer" : "testsuite_with_gibberish:'\"");
 }
